#include <string.h>
#include <stdlib.h>
#include "SDL.h"
#include "SDL_opengl.h"
#include "chipmunk.h"

#include "common.h"
#include "game.h"
#include "texture.h"
#include "font.h"
#include "sound.h"

#define MAXLEVELS 7
#define TARGETFPS 60.0f
#define GRAVITY 5
#define MAXSPEED 5
#define MAXLINES 100
#define MAXBUTTONS 10
#define PLAYERSIZE 50

typedef struct {
	float x;
	float y;
} Point;

typedef struct {
	Point P1;
	Point P2;
	int Active;
} Line;

typedef struct {
	Point loc;
	Point vel;
	Point accel;
	int frame;
	float framecount;
	float size;
	float targetsize;
	int facing;
	int movingright;
	int movingleft;
	int jumping;
	TextureRef texture;
	cpBody *body;
	cpShape *shape;
} Player;

typedef struct {
	Point loc;
	float size;
	float alpha;
} Box;

typedef struct {
	int Active;
	int butset;
	Point butloc;
	int doorset;
	Point doorloc;
	int tripped;
	float openPercent;
	cpBody *doorbody;
	cpShape *doorshape;
} Button;

// -- chipmunk stuff
cpSpace *space;
cpBody *staticBody;

Box selectionBox;
Player P1, P2;
Line Lines[MAXLINES];
int playing = TRUE;
int twoplayer;
int selectedPlayer;
int makeJump = FALSE;
int targetFPS = (1000.0f / TARGETFPS);
int sleeplength = 0;
double deltaTime = 0.0;
int thisTime = 0;
int lastTime = 0;
int CurrentLevel = 0;
char title[100];
TextureRef items;
Point LevelEnd;
Button Buttons[MAXBUTTONS];

void LoadGraphics()
{
	// player graphic
	P1.texture = GetTexture("gfx/sphereguy.tga", 0);
	P2.texture = GetTexture("gfx/legguy.tga", 0);
	items = GetTexture("gfx/objects.tga", 0);
	// box graphic
	//BoxTexture = GetTexture("gfx/boxes.tga", 0);
	
	return;
}

void LoadSounds()
{
	//LoadSound("snd/found-robot.ogg");
	
	return;
}

void AddLine(X1, Y1, X2, Y2)
{
	int i;
	
	for (i = 0; i < MAXLINES; i++)
	{
		if (Lines[i].Active == FALSE)
		{
			Lines[i].Active = TRUE;
			Lines[i].P1.x = X1;
			Lines[i].P1.y = Y1;
			Lines[i].P2.x = X2;
			Lines[i].P2.y = Y2;
			
			return;
		}
	}
	
	sys_log("Couldn't add a line!\n");
	
	return;
}

void LoopStart()
{
	thisTime = SDL_GetTicks();
	deltaTime = (double)(thisTime - lastTime) / 1000.0f;
	lastTime = thisTime;
	
	return;
}

void LoopEnd()
{
	sleeplength = targetFPS - (SDL_GetTicks() - thisTime);
    if (sleeplength < 0) sleeplength = 0;
    SDL_Delay(sleeplength); // wait so we don't use up all CPUs! LOLS
    
    return;
}


void ShowIntroScreen()
{
	float time = 3.0f;
	SDL_Event event;
	TextureRef background = GetTexture("gfx/menubg.tga", 0);
	char buf [100];
	char timebuf [100];
	
	thisTime = SDL_GetTicks();
	lastTime = SDL_GetTicks();
	deltaTime = 0;
	
	if (CurrentLevel != (MAXLEVELS + 1))
	{
		sprintf(buf, "Level %d - %s", CurrentLevel, title);
	}
	else
	{
		sprintf(buf, "%s", title);
	}
	
	while (time > 0)
	{
		LoopStart();
		
		// Inputs first
		while ( SDL_PollEvent( &event ) )
	    {
	        switch( event.type )
	        {
	        	case SDL_JOYBUTTONDOWN:
	        		if (twoplayer)
	        		{
	        			time = 0.0f;     			
	        		}
	        		break;
	        	case SDL_KEYDOWN:
		    		switch (event.key.keysym.sym)
					{
						case SDLK_SPACE:
						case SDLK_z:
						case SDLK_LCTRL:
						case SDLK_x:
						case SDLK_ESCAPE:
							time = 0.0f;
							break;
							
						default:
							break;
					}
	        		break;
	        	case SDL_QUIT:
	        		Quit(1);
	                break;
	        	default:
	        	    break;
	        }
	    }   
		
		// it's draw time!
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer
		glLoadIdentity();
		
		// Show background
		glBindTexture(GL_TEXTURE_2D, background.textureID);
				
			glBegin(GL_QUADS);
					glTexCoord2f(1.0f, 0.0f);
					glVertex3i(1024.0f, 0.0f, 0.0f);
					glTexCoord2f(0.0f, 0.0f);
					glVertex3i(0.0f, 0.0f, 0.0f);
					glTexCoord2f(0.0f, 1.0f);
					glVertex3i(0.0f, 512.0f, 0.0f);
					glTexCoord2f(1.0f, 1.0f);
					glVertex3i(1024.0f, 512.0f, 0);
			glEnd();
		
		
		DrawFontText(buf, 180, 129, FALSE);
		sprintf(timebuf, "%f", time);
		if (CurrentLevel != (MAXLEVELS + 1))
			DrawFontText(timebuf, 200, 260 - (time * 20), FALSE);
		
		// buffaswap
		SDL_GL_SwapBuffers();
		
		time -= deltaTime;
		LoopEnd();
	}
	
	
}

void LoadMap(int MapNum)
{
	int i;
	// Clear map data
	for (i = 0; i < MAXLINES; i++)
	{
		Lines[i].Active = FALSE;
	}
	
	for (i = 0; i < MAXBUTTONS; i++)
	{
		Buttons[i].Active = FALSE;
		Buttons[i].doorset = FALSE;
		Buttons[i].butset = FALSE;
	}
	
	
	// Add default lines
	Lines[0].Active = TRUE;
	Lines[0].P1.x = 10.0f;
	Lines[0].P1.y = 30.0f;
	Lines[0].P2.x = 10.0f;
	Lines[0].P2.y = ScreenHeight - 10.0f;
	Lines[1].Active = TRUE;
	Lines[1].P1.x = 10.0f;
	Lines[1].P1.y = ScreenHeight - 10.0f;
	Lines[1].P2.x = ScreenWidth - 10.0f;
	Lines[1].P2.y = ScreenHeight - 10.0f;
	Lines[2].Active = TRUE;
	Lines[2].P1.x = ScreenWidth - 10.0f;
	Lines[2].P1.y = ScreenHeight - 10.0f;
	Lines[2].P2.x = ScreenWidth - 10.0f;
	Lines[2].P2.y = 30.0f;
	Lines[3].Active = TRUE;
	Lines[3].P1.x = ScreenWidth - 10.0f;
	Lines[3].P1.y = 30.0f;
	Lines[3].P2.x = 10.0f;
	Lines[3].P2.y = 30.0f;
	
	// open up the mapfile 
	char filename[50];
	sprintf(filename, "lvl/%d.txt", MapNum);
	sys_log("Opening Level: %s\n", filename);
	
	
	FILE *file = fopen(filename, "r");
	if ( file )
	{
		  char line [ BUFSIZ ];
		  while ( fgets(line, sizeof line, file) )
		  {
			 char * split;
			 split = strtok(line, ":");
			 while (split != NULL)
			 {
				 //sys_log("%s\n", split);
				 
					if (strcmp(split,"T") == 0) // Title of the level
					{
						split = strtok(NULL,":");
						strcpy(title, split);
		
						break;
					} 
					else if (strcmp(split, "P") == 0) // Player location
					{
						split = strtok(NULL,":");
						char *coord;
						int temp;
						if (strcmp(split, "1") == 0) // first player
						{
							coord = strtok(NULL, ":");
							
							temp = atoi(strtok(coord, ","));
							P1.loc.x = (float)temp;
							temp = atoi(strtok(NULL, ","));
							P1.loc.y = (float)temp;
							break;
						}
						else if (strcmp(split, "2") == 0) // 2nd player
						{
							coord = strtok(NULL, ":");
							
							temp = atoi(strtok(coord, ","));
							P2.loc.x = (float)temp;
							temp = atoi(strtok(NULL, ","));
							P2.loc.y = (float)temp;
							break;
						}
					}
					else if (strcmp(split,"L") == 0) // Level Line
					{
						int X1, Y1, X2, Y2;
						char *one;
						char *two;
						char *temp;
						
						one = strtok(NULL,":");
						two = strtok(NULL,":");
						
						temp = strtok(one, ",");
						X1 = atoi(temp);
						temp = strtok(NULL, ",");
						Y1 = atoi(temp);
						
						temp = strtok(two, ",");
						X2 = atoi(temp);
						temp = strtok(NULL, ",");
						Y2 = atoi(temp);
						
						AddLine(X1, Y1, X2, Y2);
						//sys_log("Line: %d,%d %d,%d\n", X1, Y1, X2, Y2);
						break;
					}
					else if (strcmp(split, "B") == 0) // Button
					{
						split = strtok(NULL,":");
						int ID = atoi(split);
						ID = ID - 1; // zero based array
						
						char *one;
						char *temp;
						
						one = strtok(NULL,":");
						
						temp = strtok(one, ",");
						Buttons[ID].butloc.x = (float)atoi(temp);
						temp = strtok(NULL, ",");
						Buttons[ID].butloc.y = (float)atoi(temp);
						
						Buttons[ID].butset = TRUE;
						break;
					}
					else if (strcmp(split, "D") == 0) // Door
					{
						split = strtok(NULL,":");
						int ID = atoi(split);
						ID = ID - 1; // zero based array
						
						char *one;
						char *temp;
						
						one = strtok(NULL,":");
						
						temp = strtok(one, ",");
						Buttons[ID].doorloc.x = (float)atoi(temp);
						temp = strtok(NULL, ",");
						Buttons[ID].doorloc.y = (float)atoi(temp);
						Buttons[ID].openPercent = 100.0f;
						Buttons[ID].doorset = TRUE;
					}
					else if (strcmp(split, "E") == 0) // Level Exit
					{
						split = strtok(NULL,":");
						char *temp;
						temp = strtok(split, ",");
						LevelEnd.x = (float)atoi(temp);
						temp = strtok(NULL, ",");
						LevelEnd.y = (float)atoi(temp);
						
						//sys_log("LevelEnd: %f, %f\n", LevelEnd.x, LevelEnd.y);
						
						break;
					}
					break;
				}
				 
				
		  }
		  
		 // Check that all button/door combinations are valid!
		 int b;
		 for (b = 0; b < MAXBUTTONS; b++)
		 {
			 if (Buttons[b].butset == TRUE && Buttons[b].doorset == TRUE)
			 {
				 Buttons[b].Active = TRUE;
			 }
		 }
		  
	}
	else
	{
		sys_log("Can't open mapfile: %s", filename);
		Quit(2);
	}
	
	ShowIntroScreen();
	
	return;
}

void HandleInputs()
{
	SDL_Event event;

	    while ( SDL_PollEvent( &event ) )
	    {
	        switch( event.type )
	        {
	        	case SDL_JOYAXISMOTION:
	        		if (twoplayer && event.jaxis.axis == 0)
	        		{
	        			if (event.jaxis.value > 500)
	        			{
	        				P2.movingright = TRUE;
	        				P2.movingleft = FALSE;
	        			}
	        			else if (event.jaxis.value < -500)
	        			{
	        				P2.movingleft = TRUE;
	        				P2.movingright = FALSE;
	        			}
	        			else
	        			{
	        				P2.movingright = FALSE;
	        				P2.movingleft = FALSE;
	        			}
	        		}
	        		break;
	        	case SDL_JOYBUTTONDOWN:
	        		if (twoplayer)
	        		{
	        			P2.jumping = TRUE;        			
	        		}
	        		break;
	        	case SDL_JOYBUTTONUP:
	        		if (twoplayer)
	        		{
	        			P2.jumping = FALSE;
	        		}
	        		break;
	        	case SDL_KEYDOWN:
		    		switch (event.key.keysym.sym)
					{
						//case SDLK_UP:
						case SDLK_LEFT:
							if (selectedPlayer == 1)
								P1.movingleft = TRUE;
							else
								P2.movingleft = TRUE;
												
							break;
						//case SDLK_DOWN:
						case SDLK_RIGHT:
							if (selectedPlayer == 1)
								P1.movingright = TRUE;
							else
								P2.movingright = TRUE;
							break;
						case SDLK_UP:
							
							break;
						case SDLK_SPACE:
						case SDLK_z:
						case SDLK_LCTRL:
							if (selectedPlayer == 1)
							{
								P1.targetsize = (PLAYERSIZE * 0.5);
								PlaySound("snd/sizedown.ogg");
							}
							else
								P2.jumping = TRUE;
						
							break;
						case SDLK_x:
							if (twoplayer == FALSE)
							{
								// switch player
								if (selectedPlayer == 1)
								{
									if (P1.movingleft == TRUE)
										P2.movingleft = TRUE;

									if (P1.movingright == TRUE)
										P2.movingright = TRUE;
									
									P1.movingleft = FALSE;
									P1.movingright = FALSE;
									selectedPlayer = 2;
									selectionBox.loc.x = P2.loc.x + (PLAYERSIZE * 0.5);
									selectionBox.loc.y = P2.loc.y + (PLAYERSIZE * 0.5);
								}
								else
								{
									if (P2.movingleft == TRUE)
										P1.movingleft = TRUE;

									if (P2.movingright == TRUE)
										P1.movingright = TRUE;
									
									P2.movingleft = FALSE;
									P2.movingright = FALSE;
									selectedPlayer = 1;
									selectionBox.loc.x = P1.loc.x + (PLAYERSIZE * 0.5);
									selectionBox.loc.y = P1.loc.y + (PLAYERSIZE * 0.5);
								}
								
								selectionBox.size = 30;
								selectionBox.alpha = 1.00f;
							}
							break;
						case SDLK_ESCAPE:
							// TODO: "are you sure?"
							playing = FALSE;
						default:
							break;
					}
	        		break;
	        	case SDL_KEYUP:
	        		switch (event.key.keysym.sym)
					{
						case SDLK_LEFT:
							if (selectedPlayer == 1)
								P1.movingleft = FALSE;
							else
								P2.movingleft = FALSE;
								
							break;
						case SDLK_RIGHT:
							if (selectedPlayer == 1)
								P1.movingright = FALSE;
							else
								P2.movingright = FALSE;
								
							break;
						case SDLK_z:
							if (selectedPlayer == 1)
							{
								P1.targetsize = (PLAYERSIZE);
								PlaySound("snd/sizeup.ogg");
							}
							else
								P2.jumping = FALSE;

						default:
							break;
					}
	        		break;	
	        	case SDL_QUIT:
	        		Quit(1);
	                break;
	        	default:
	        	    break;
	        }
	    }   
	    
	
	return;
}

void DrawLines()
{
	int i;
	// First the "glowy bits"
	glDisable(GL_TEXTURE_2D);
	
	glLineWidth(5.5f);
	glColor4f(1.0f, 1.0f, 1.0f, 0.3f);
	for (i = 0; i < MAXLINES; i++)
	{
		if (Lines[i].Active == TRUE)
		{
			// Draw it!
			//glBegin(GL_LINES);
			//	glVertex3f(Lines[i].P1.x, Lines[i].P1.y, 0.0f);
			//	glVertex3f(Lines[i].P2.x, Lines[i].P2.y, 0.0f);
			//glEnd();
			
			//sys_log("Line: %f, %f %f, %f\n", Lines[i].P1.x, Lines[i].P1.y, Lines[i].P2.x, Lines[i].P2.y);
		}
	}
	//sys_log("\n");
	
	glLineWidth(1.0f);
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	// Next the actual lines
	for (i = 0; i < MAXLINES; i++)
	{
		if (Lines[i].Active == TRUE)
		{
			// Draw it!
			glBegin(GL_LINES);
				glVertex3f(Lines[i].P1.x, Lines[i].P1.y, 0.0f);
				glVertex3f(Lines[i].P2.x, Lines[i].P2.y, 0.0f);
			glEnd();
		}
	}
	glEnable(GL_TEXTURE_2D);
}

void DrawObjects()
{
	float pixelx;
	float spritex;
	
	pixelx = 1 / (float)(items.width);
	spritex = pixelx * 64;
	
	// Draw exit
	glBindTexture(GL_TEXTURE_2D, items.textureID);
	glPushMatrix();
		
	glBegin(GL_QUADS);
		glTexCoord2f(spritex, 0.0f);
		glVertex3i(LevelEnd.x + PLAYERSIZE, LevelEnd.y, 0.0f);
			
		glTexCoord2f(0.0f, 0.0f);
		glVertex3i(LevelEnd.x, LevelEnd.y, 0.0f);
		
		glTexCoord2f(0.0f, 1.0f);
		glVertex3i(LevelEnd.x, LevelEnd.y + PLAYERSIZE, 0.0f);
		
		glTexCoord2f(spritex, 1.0f);
		glVertex3i(LevelEnd.x + PLAYERSIZE, LevelEnd.y + PLAYERSIZE, 0.0f);
	glEnd();
	
	// -- button
	
	int i;
	float tx;
	for (i = 0; i < MAXBUTTONS; i++)
	{
		if (Buttons[i].Active == TRUE)
		{
			if (Buttons[i].tripped == TRUE)
				tx = spritex * 3;
			else
				tx = spritex * 2;
			
			glBegin(GL_QUADS);
				glTexCoord2f(tx + spritex, 0.0f);
				glVertex3i(Buttons[i].butloc.x + PLAYERSIZE, Buttons[i].butloc.y, 0.0f);
					
				glTexCoord2f(tx , 0.0f);
				glVertex3i(Buttons[i].butloc.x, Buttons[i].butloc.y, 0.0f);
				
				glTexCoord2f(tx , 1.0f);
				glVertex3i(Buttons[i].butloc.x, Buttons[i].butloc.y + PLAYERSIZE, 0.0f);
				
				glTexCoord2f(tx + spritex, 1.0f);
				glVertex3i(Buttons[i].butloc.x + PLAYERSIZE, Buttons[i].butloc.y + PLAYERSIZE, 0.0f);
			glEnd();
			
			// Draw door if not open
			if (Buttons[i].openPercent > 0.0f)
			{
				// draw door
				glDisable(GL_TEXTURE_2D);
				glColor4f(0.7f, 0.7f, 0.7f, 1.0f);
				
				float height = (PLAYERSIZE * 2) * (Buttons[i].openPercent * 0.01);
				
				glBegin(GL_QUADS);
					glVertex3i(Buttons[i].doorloc.x + PLAYERSIZE - 12.0f, Buttons[i].doorloc.y + PLAYERSIZE - height, 0.0f);
					glVertex3i(Buttons[i].doorloc.x + 12.0f, Buttons[i].doorloc.y + PLAYERSIZE - height, 0.0f);
					glVertex3i(Buttons[i].doorloc.x + 12.0f, Buttons[i].doorloc.y + PLAYERSIZE, 0.0f);
					glVertex3i(Buttons[i].doorloc.x + PLAYERSIZE - 12.0f, Buttons[i].doorloc.y + PLAYERSIZE, 0.0f);
				glEnd();
				
				glEnable(GL_TEXTURE_2D);
				glColor4f(1.f, 1.0f, 1.0f, 1.0f);
			}
			
			tx = spritex;
			
			glBegin(GL_QUADS);
				glTexCoord2f(tx + spritex, 0.0f);
				glVertex3i(Buttons[i].doorloc.x + PLAYERSIZE, Buttons[i].doorloc.y, 0.0f);
					
				glTexCoord2f(tx , 0.0f);
				glVertex3i(Buttons[i].doorloc.x, Buttons[i].doorloc.y, 0.0f);
				
				glTexCoord2f(tx , 1.0f);
				glVertex3i(Buttons[i].doorloc.x, Buttons[i].doorloc.y + PLAYERSIZE, 0.0f);
				
				glTexCoord2f(tx + spritex, 1.0f);
				glVertex3i(Buttons[i].doorloc.x + PLAYERSIZE, Buttons[i].doorloc.y + PLAYERSIZE, 0.0f);
			glEnd();
			
		}
	}
	
	// -- selection box
	if (selectionBox.alpha > 0.0f)
	{
		glDisable(GL_TEXTURE_2D);
		
		glLineWidth(6.0f);
		glColor4f(1.0f, 1.0f, 1.0f, selectionBox.alpha);
		
		glBegin(GL_LINES);
			glVertex3f(selectionBox.loc.x - selectionBox.size, selectionBox.loc.y - selectionBox.size, 0.0f);
			glVertex3f(selectionBox.loc.x - selectionBox.size, selectionBox.loc.y + selectionBox.size, 0.0f);
			
			glVertex3f(selectionBox.loc.x - selectionBox.size, selectionBox.loc.y - selectionBox.size, 0.0f);
			glVertex3f(selectionBox.loc.x + selectionBox.size, selectionBox.loc.y - selectionBox.size, 0.0f);
				
			glVertex3f(selectionBox.loc.x + selectionBox.size, selectionBox.loc.y - selectionBox.size, 0.0f);
			glVertex3f(selectionBox.loc.x + selectionBox.size, selectionBox.loc.y + selectionBox.size, 0.0f);
		
			glVertex3f(selectionBox.loc.x - selectionBox.size, selectionBox.loc.y + selectionBox.size, 0.0f);
			glVertex3f(selectionBox.loc.x + selectionBox.size, selectionBox.loc.y + selectionBox.size, 0.0f);
												
		glEnd();
		
		glEnable(GL_TEXTURE_2D);		
	}
	
	return;
}
void DrawPlayers()
{	
	int framey = 0;
	float tx, ty;
	float pixelx, pixely;
	float spritex, spritey;
	
	glEnable(GL_TEXTURE_2D);
	
	// ------------------- player 1
	
	pixelx = 1 / (float)(P1.texture.width);
	pixely = 1 / (float)(P1.texture.height);
	spritex = pixelx * 64;
	spritey = pixely * 64;
	
	tx = ((float)P1.frame * spritex);
	ty = ((float)framey * spritey);
	
	//fprintf(stdout, "%f, %f, %f, %f\n", tx, ty, tx + spritex, ty + spritey);
	glBindTexture(GL_TEXTURE_2D, P1.texture.textureID);
	glPushMatrix();
	
	glBegin(GL_QUADS);
		if (!P1.facing)
			glTexCoord2f(tx, ty);
		else
			glTexCoord2f(tx + spritex, ty);
		glVertex3i(P1.loc.x + (float)P1.size, P1.loc.y, 0.0f);
		
		if (!P1.facing)
			glTexCoord2f(tx + spritex, ty);
		else
			glTexCoord2f(tx, ty);
		glVertex3i(P1.loc.x, P1.loc.y, 0.0f);
		
		if (!P1.facing)
			glTexCoord2f(tx + spritex, ty + spritey);
		else
			glTexCoord2f(tx, ty + spritey);
		glVertex3i(P1.loc.x, P1.loc.y + (float)P1.size, 0.0f);
		
		if (!P1.facing)
			glTexCoord2f(tx, ty + spritey);
		else
			glTexCoord2f(tx + spritex, ty + spritey);
		glVertex3i(P1.loc.x + (float)P1.size, P1.loc.y + (float)P1.size, 0.0f);
	glEnd();
	
	// -------------player 2
	
	pixelx = 1 / (float)(P2.texture.width);
	pixely = 1 / (float)(P2.texture.height);
	spritex = pixelx * 64;
	spritey = pixely * 64;
	
	tx = ((float)P2.frame * spritex);
	ty = ((float)framey * spritey);
	
	//fprintf(stdout, "%f, %f, %f, %f\n", tx, ty, tx + spritex, ty + spritey);
	glBindTexture(GL_TEXTURE_2D, P2.texture.textureID);
	glPushMatrix();
	
	glBegin(GL_QUADS);
		if (!P2.facing)
			glTexCoord2f(tx, ty);
		else
			glTexCoord2f(tx + spritex, ty);
		glVertex3i(P2.loc.x + (float)P2.size, P2.loc.y, 0.0f);
		
		if (!P2.facing)
			glTexCoord2f(tx + spritex, ty);
		else
			glTexCoord2f(tx, ty);
		glVertex3i(P2.loc.x, P2.loc.y, 0.0f);
		
		if (!P2.facing)
			glTexCoord2f(tx + spritex, ty + spritey);
		else
			glTexCoord2f(tx, ty + spritey);
		glVertex3i(P2.loc.x, P2.loc.y + (float)P2.size, 0.0f);
		
		if (!P2.facing)
			glTexCoord2f(tx, ty + spritey);
		else
			glTexCoord2f(tx + spritex, ty + spritey);
		glVertex3i(P2.loc.x + (float)P2.size, P2.loc.y + (float)P2.size, 0.0f);
	glEnd();
	
	return;
}

void Draw()
{
	//clear buffarz
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer
	glLoadIdentity();	
		
	DrawLines();
	DrawPlayers();
	DrawObjects();	
	// Level Title
	char buf[100];
	sprintf(buf, "Level %d - %s", CurrentLevel, title);
	DrawFontText(buf, 20, 10, FALSE);
	
	return;
}

int CheckButton(Point One, Point Two)
{
	//(abs(One.y - Two.y) < PLAYERSIZE) || 
	if ((abs(One.y - Two.y) <= PLAYERSIZE))
	{
		//sys_log("collcheck: %f, %f  %f, %f\n", One.x, One.y, Two.x, Two.y);
		//(abs(One.x - Two.x) < PLAYERSIZE) ||
		//(PLAYERSIZE) -
		if ((abs(One.x - Two.x) <= PLAYERSIZE))
		{
			//sys_log("collided!\n");
			return TRUE;
		}
	}
	
	return FALSE;
}

static int
collFunc(cpShape *a, cpShape *b, cpContact *contacts, int numContacts, cpFloat normal_coef, void *data)
{
	//int *some_ptr = (int *)data;
	
	if (P2.jumping && (a == P2.shape || b == P2.shape))
	{
		makeJump = TRUE;
		P2.jumping = FALSE;
	}
	
	return 1;
}

void ComputeAll(double dt)
{
	
	cpBodyResetForces(P1.body);
	cpBodyResetForces(P2.body);
	
	if (P1.targetsize != P1.size)
	{
		if (P1.targetsize > P1.size)
		{
			P1.size += (PLAYERSIZE * 0.05);
			if (P1.size > PLAYERSIZE)
				P1.size = PLAYERSIZE;
		}
		else
		{
			P1.size -= (PLAYERSIZE * 0.05);
			if (P1.size < (PLAYERSIZE * 0.5))
				P1.size = (PLAYERSIZE * 0.5);
		}
		
		P1.body->m = cpMomentForCircle(1.0f, P1.size * 0.5, 0.0f, cpvzero);
		
		// This is kinda dumb... *shakes a fist at Chipmunk*
		cpSpaceRemoveShape(space, P1.shape);
		
		P1.shape = cpCircleShapeNew(P1.body, P1.size * 0.5, cpvzero);
		P1.shape->e = 0.0; P1.shape->u = 1.0;
		cpSpaceAddShape(space, P1.shape);
	}
	
	if (P1.movingleft)
		P1.body->f.x -= 150.0f;
	if (P1.movingright)
		P1.body->f.x += 150.0f;
		
	if (P2.movingleft)
		P2.body->f.x -= 150.0f;
	if (P2.movingright)
		P2.body->f.x += 150.0f;
	
	if (makeJump)
	{
		P2.body->f.y -= 8000.0f;
		makeJump = FALSE;
		PlaySound("snd/jump.ogg");
	}
	
	// p2 stuff
	//if (P2.body->v.x < 100 && P2.body->v.x > -100 && P2.body->f.x < 10 && P2.body->f.x > -10)
	if (!P2.movingleft && !P2.movingright)
		P2.body->v.x = P2.body->v.x * 0.9;
	
	cpSpaceStep(space, dt);
	
	// assign locations
	//cpVect v = cpvadd(P1.body->p, 0.0f);
	P1.loc.x = P1.body->p.x - (P1.size * 0.5);
	P1.loc.y = P1.body->p.y - (P1.size * 0.5);
	
	//v = cpvadd(P2.body->p, 0.0f);
	P2.loc.x = P2.body->p.x - (P2.size * 0.5);
	P2.loc.y = P2.body->p.y - (P2.size * 0.5);
	
	// set frame
	P1.frame = ((cpvtoangle(P1.body->rot) * (180.0/M_PI))/ 16.0f);
	P2.frame = ((cpvtoangle(P2.body->rot) * (180.0/M_PI))/ 16.0f);
	
	if (P1.body->v.x > 0)
	{
		P1.facing = TRUE;
		P1.frame = 16 - P1.frame;
	}
	else
		P1.facing = FALSE;
	
	if (P2.body->v.x > 0)
	{
		P2.facing = TRUE;
		P2.frame = 16 - P2.frame;
	}
	else
		P2.facing = FALSE;
	
	// check if a player tripped a button
	// door
	int i;
	for (i = 0; i < MAXBUTTONS; i++)
	{
		if (Buttons[i].Active == TRUE)
		{
			if (Buttons[i].tripped == FALSE)
			{
				if ((CheckButton(P2.loc, Buttons[i].butloc) == TRUE || CheckButton(P1.loc, Buttons[i].butloc) == TRUE))
				{
					//sys_log("Button trip\n");
					Buttons[i].tripped = TRUE;
					cpSpaceRemoveShape(space, Buttons[i].doorshape);
				}
			}
			else
			{
				if ((CheckButton(P2.loc, Buttons[i].butloc) == FALSE) && (CheckButton(P1.loc, Buttons[i].butloc) == FALSE))
				{
					//sys_log("Button un-trip\n");
					Buttons[i].tripped = FALSE;
					cpSpaceAddShape(space, Buttons[i].doorshape);
				}
			}
			
			if (Buttons[i].tripped == TRUE && Buttons[i].openPercent != 0.0f)
				Buttons[i].openPercent -= 5.0f;
			else if (Buttons[i].tripped == FALSE && Buttons[i].openPercent != 100.0f)
				Buttons[i].openPercent += 5.0f;
		}
	}
	
	//selection box
	if (selectionBox.alpha > 0.0f)
	{
		selectionBox.alpha -= 0.05f;
		selectionBox.size += 3;
	}
	
	return;
}


// whaat? more chipmunk craziness
int some_value = 42;

void ResetStuff()
{
	playing = TRUE;
	P1.size = PLAYERSIZE;
	P1.targetsize = PLAYERSIZE;
	P1.jumping = FALSE;
	P2.size = PLAYERSIZE;
	P2.jumping = FALSE;
	P1.movingleft = FALSE;
	P1.movingright = FALSE;
	P2.movingleft = FALSE;
	P2.movingright = FALSE;
	selectedPlayer = 1;
	
	selectionBox.alpha = 1.0f;
	selectionBox.loc.x = P1.loc.x + (PLAYERSIZE * 0.5);
	selectionBox.loc.y = P1.loc.y + (PLAYERSIZE * 0.5);
	selectionBox.size = 30;
	
	//sys_log("P1: %f, %f\nP2: %f, %f", P1.loc.x, P1.loc.y, P2.loc.x, P2.loc.y);
	
	// -- Reset chipmunk stuff
	if (space && staticBody)
	{	
		cpSpaceFreeChildren(space);
		cpSpaceFree(space);
		
		cpBodyFree(staticBody);
	}
	
	staticBody = cpBodyNew(INFINITY, INFINITY);
		
	cpResetShapeIdCounter();
	space = cpSpaceNew();
	//cpSpaceResizeStaticHash(space, 30.0f, 999);
	//cpSpaceResizeActiveHash(space, 50.0f, 100);
	space->gravity = cpv(0.0f, 130.0f);
	
	//cpBody *body;
	
	cpShape *shape;
	
	// add lines
	int i;
	
	for (i = 0; i < MAXLINES; i++)
	{
		if (Lines[i].Active == TRUE)
		{
			cpVect one;
			cpVect two;
			
			one.x = Lines[i].P1.x;
			one.y = Lines[i].P1.y;
			two.x = Lines[i].P2.x;
			two.y = Lines[i].P2.y;
			
			shape = cpSegmentShapeNew(staticBody, one, two, 1.0f);
			shape->e = 1.0; 
			shape->u = 1.0;
			cpSpaceAddStaticShape(space, shape);
		}
	}
	
	//add players
	
	cpFloat mass = 1.0f;
	cpFloat moment = cpMomentForCircle(mass, P1.size * 0.5, 0.0f, cpvzero);
	
	P1.body = cpBodyNew(mass, moment);
	P1.body->p.x = P1.loc.x;
	P1.body->p.y = P1.loc.y;
	P2.body = cpBodyNew(mass, moment);
	P2.body->p.x = P2.loc.x;
	P2.body->p.y = P2.loc.y;
	
	cpSpaceAddBody(space, P1.body);
	cpSpaceAddBody(space, P2.body);
	
	P1.shape = cpCircleShapeNew(P1.body, P1.size * 0.5, cpvzero);
	P1.shape->e = 0.0; P1.shape->u = 1.0;
	cpSpaceAddShape(space, P1.shape);
	
	P2.shape = cpCircleShapeNew(P2.body, P2.size * 0.5, cpvzero);
	P2.shape->e = 0.0; P2.shape->u = 5.5;
	cpSpaceAddShape(space, P2.shape);
	
	cpSpaceSetDefaultCollisionPairFunc(space, &collFunc, &some_value);
	

	// Add shapes for all doors
	for (i = 0; i < MAXBUTTONS; i++)
	{
		if (Buttons[i].Active == TRUE)
		{
			cpVect verts[] = {
				cpv(Buttons[i].doorloc.x + PLAYERSIZE - 12, Buttons[i].doorloc.y + PLAYERSIZE),
				cpv(Buttons[i].doorloc.x + PLAYERSIZE - 12, Buttons[i].doorloc.y - PLAYERSIZE),
				cpv(Buttons[i].doorloc.x + 12, Buttons[i].doorloc.y - PLAYERSIZE),
				cpv(Buttons[i].doorloc.x + 12, Buttons[i].doorloc.y + PLAYERSIZE),
			};
			
			//Buttons[i].doorbody = cpBodyNew(1.0, cpMomentForPoly(1.0, 4, verts, cpvzero));
			/*if (Buttons[i].doorshape)
			{
				cpShapeDestroy(Buttons[i].doorshape);
			}*/
			
			Buttons[i].doorshape = cpPolyShapeNew(staticBody, 4, verts, cpvzero);
			Buttons[i].doorshape->e = 0.0;
			Buttons[i].doorshape->u = 1.5;
			Buttons[i].doorshape->collision_type = 1;
			cpSpaceAddShape(space, Buttons[i].doorshape);
		}
	}
	
	thisTime = SDL_GetTicks();
	lastTime = SDL_GetTicks();
	deltaTime = 0;
	
	return;
}

void CheckForWin()
{
	// check win condition
	if (CheckButton(P2.loc, LevelEnd) == TRUE || CheckButton(P1.loc, LevelEnd) == TRUE)
	{
		PlaySound("snd/win.ogg");
		CurrentLevel++;
		if (CurrentLevel == MAXLEVELS + 1)
		{
			strcpy(title, "YOU WIN! You make a good team!");
			ShowIntroScreen();
			playing = FALSE;
			//Quit(1);
		}
		else
		{	
			LoadMap(CurrentLevel);
			ResetStuff();
		}
	}
}

void StartGame(int players)
{
	if (players == 1)
		twoplayer = FALSE;
	else
		twoplayer = TRUE;
	
	LoadSounds();
	LoadGraphics();
	InitFont();
	CurrentLevel = 1;
	LoadMap(CurrentLevel);	
	ResetStuff();

	while(playing)
	{
		LoopStart();
		
		HandleInputs();
		
		ComputeAll(deltaTime);
		
		Draw();
		SDL_GL_SwapBuffers();
		
		CheckForWin();
		
		LoopEnd();
		
	}
	return;
}
