#include <time.h>
#include "SDL.h"
#include "SDL_opengl.h"
#include "physfs.h"

#include "chipmunk.h"

#include "common.h"
#include "sound.h"
#include "menu.h"
#include "game.h"

char* version = "0.1";
int fullscreen = FALSE;
SDL_Surface *surface;

void InitGlobals()
{
	ScreenWidth = 800;
	ScreenHeight = 480;
	
	return;
}

void InitPhysFS(char *arg)
{
	// Get PhysFS stuff going
	if (!PHYSFS_init(arg))
		fprintf(stderr, PHYSFS_getLastError());
	if (PHYSFS_setSaneConfig("fydo", "LD11", "ZIP", 0, 0) == 0)
		fprintf(stderr, PHYSFS_getLastError());
	
	return;
}

void InitOpenGL()
{
	// viewport / ortho stuff
	glViewport(0, 0, ScreenWidth, ScreenHeight);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0.0f, ScreenWidth, ScreenHeight, 0.0f, -1.0f, 1.0f);
	glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
	    
    // texturing
    glEnable(GL_TEXTURE_2D);
    glEnable(GL_BLEND);
    
    // smooth shading
    glShadeModel( GL_SMOOTH );

    // background color
    glClearColor( 0.164f, 0.164f, 0.164f, 1.0f );

    // no depth testing
    glDisable( GL_DEPTH_TEST );
 
    // yay alphablends
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

    // don't draw faces we can't see :)
    glEnable(GL_CULL_FACE);
    
    // sure, why not?
    glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
	
	return;
}

void InitSDL(int fullscreen)
{
	sys_log("Initializing SDL ... ");
	
	//Flags to pass to SDL_SetVideoMode
	int videoFlags;
	const SDL_VideoInfo *videoInfo;
	
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_JOYSTICK) < 0)
    {
		sys_log("\nVideo initialization failed: %s\n", SDL_GetError());
        Quit(2);
    }
    else
    {
    	sys_log("OK!\n");
    }

    videoInfo = SDL_GetVideoInfo();

    if (!videoInfo)
    {
    	sys_log("Video query failed: %s\n", SDL_GetError());
        Quit(2);
    }

    videoFlags  = SDL_OPENGL;          /* Enable OpenGL in SDL */
    videoFlags |= SDL_GL_DOUBLEBUFFER; /* Enable double buffering */
    videoFlags |= SDL_HWPALETTE;       /* Store the palette in hardware */
    //videoFlags |= SDL_RESIZABLE;       /* Enable window resizing */

    if ( videoInfo->hw_available )
        videoFlags |= SDL_HWSURFACE;
    else
        videoFlags |= SDL_SWSURFACE;

    /* This checks if hardware blits can be done */
    if ( videoInfo->blit_hw )
        videoFlags |= SDL_HWACCEL;

    if (fullscreen) videoFlags |= SDL_FULLSCREEN;

    //double buffering
    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, TRUE );
    //vsync
    //SDL_GL_SetAttribute(SDL_GL_SWAP_CONTROL, TRUE);

    int ScreenBPP = videoInfo->vfmt->BitsPerPixel;

    sys_log("Setting video mode: %dx%d %dbpp ... ", ScreenWidth, ScreenHeight, ScreenBPP);

    surface = SDL_SetVideoMode(ScreenWidth, ScreenHeight, ScreenBPP, videoFlags);

    // Verify that it worked
    if ( !surface )
    {
    	sys_log("\nVideo mode set failed: %s\n", SDL_GetError( ) );
        Quit( 1 );
    }
    else
    {
    	sys_log("OK!\n");
    }

    SDL_WM_SetCaption("miniPals - fydo's LD11 entry", "kittay");
    SDL_ShowCursor(SDL_DISABLE);
	
	return;
}

void InitJoysticks()
{
	SDL_Joystick *joy;
	// Check for joysticks
	int i;
	sys_log("Number of Joysticks: %d\n", SDL_NumJoysticks());
	
	for (i = 0; i < SDL_NumJoysticks(); i++)
	{
	  // Open joystick
	  joy = SDL_JoystickOpen(i);
	  
	  if(joy)
	  {
		  sys_log("Opened Joystick %d\n", i);
		  sys_log("Name: %s\n", SDL_JoystickName(i));
		  sys_log("Number of Axes: %d\n", SDL_JoystickNumAxes(joy));
		  sys_log("Number of Buttons: %d\n", SDL_JoystickNumButtons(joy));
		  sys_log("Number of Balls: %d\n", SDL_JoystickNumBalls(joy));
	  }
	  else
		  printf("Couldn't open Joystick %d\n", i);	  
	}
}

int main( int argc, char **argv )
{
    // Welcome user!
	sys_log("Welcome to miniPals v%s - by fydo for LD11\n", version);
	sys_log("  http://fydo.net\n");
 
    // parse commandline args
    	
	// get things going
	InitGlobals();
	InitSDL(FALSE);
	InitJoysticks();
	InitSound();
	InitOpenGL();
	InitPhysFS(argv[0]);
	cpInitChipmunk();
	srand(time(NULL));
	
	// master loops
	int result = 0;
	
	while (result != 2)
	{
		result = DisplayMenu();
		
		if (result == 0)
		{
			// user wants to play
			StartGame(1);
		} 
		else if (result == 1 && SDL_NumJoysticks() > 0)
		{
			StartGame(2);
		}
		
	}
	
	Quit(1);
	
	return 0;
}
