// menu

#include "math.h"
#include "SDL.h"
#include "SDL_opengl.h"

#include "texture.h"
#include "common.h"
#include "font.h"
#include "sound.h"

#ifndef M_PI
	#define M_PI 3.14159265358979323846
#endif

#define HIGHEST 2
#define LOWEST 0

int direction;
float sineOsc;
int selection;
int done;
float FloatOffset;
float SlidePos;
int TargetPos;
TextureRef Background;

void LoadTextures()
{
	//Title = GetTexture("gfx/title.tga", 0);
	Background = GetTexture("gfx/menubg.tga", 0);
    
	// Pre-load other graphics
	GetTexture("gfx/legguy.tga", 0);
	GetTexture("gfx/sphereguy.tga", 0);
	
	LoadSound("snd/blip.ogg");
	LoadSound("snd/select.ogg");
	LoadSound("snd/jump.ogg");
	LoadSound("snd/win.ogg");
	LoadSound("snd/sizeup.ogg");
	LoadSound("snd/sizedown.ogg");
		
	return;
}

void MenuMoveUp()
{
	if (selection == LOWEST)
		return;
	
	selection--;
	TargetPos -= 36;
	
	PlaySound("snd/blip.ogg");
	
	return;
}

void MenuMoveDown()
{
	if (selection == HIGHEST)
		return;
	
	selection++;
	TargetPos += 36;
	
	PlaySound("snd/blip.ogg");
	
	return;
}

int ProcessInput() 
{
	SDL_Event e;
	
	while (SDL_PollEvent(&e)) switch (e.type)
	{
		case SDL_QUIT:
			Quit(0);
			
		case SDL_KEYDOWN:
			switch (e.key.keysym.sym)
			{
				case SDLK_UP:
				case SDLK_LEFT:
					MenuMoveUp();
					break;
				case SDLK_DOWN:
				case SDLK_RIGHT:
					MenuMoveDown();
					break;
				case SDLK_RETURN:
				case SDLK_SPACE:
				case SDLK_z:
					//Mix_PlayChannel(-1, menuSelectSound, 0);
					PlaySound("snd/select.ogg");
					done = 1; //selected
					break;
				case SDLK_ESCAPE:
					Quit(0);
				default:
					break;
			}
			break;
	}
	return 0; // not made a selection
}

void DrawBG()
{
	glBindTexture(GL_TEXTURE_2D, Background.textureID);
		
	glBegin(GL_QUADS);
			glTexCoord2f(1.0f, 0.0f);
			glVertex3i(1024.0f, 0.0f, 0.0f);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3i(0.0f, 0.0f, 0.0f);
			glTexCoord2f(0.0f, 1.0f);
			glVertex3i(0.0f, 512.0f, 0.0f);
			glTexCoord2f(1.0f, 1.0f);
			glVertex3i(1024.0f, 512.0f, 0);
	glEnd();
	
	return;
}

void DrawSelector()
{
	glDisable(GL_TEXTURE_2D);
	glPushMatrix();
	glColor4f(0.0f, 0.0f, 0.0f, 1.0f);
	
	glTranslatef(0.0f, SlidePos, 0.0f);
	
	glBegin(GL_QUADS);
		glVertex3i(375.0f, 125.0f, 0.0f);
		glVertex3i(175.0f, 125.0f, 0.0f);
		glVertex3i(175.0f, 150.0f, 0.0f);
		glVertex3i(375.0f, 150.0f, 0.0f);
	glEnd();
	
	glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
	glPopMatrix();
	glEnable(GL_TEXTURE_2D);
	return;
}

void DrawItems()
{
	DrawFontText("1P", 180, 129, FALSE);
	if (SDL_NumJoysticks() > 0)
		DrawFontText("2P", 180, 165, FALSE);
	else
		DrawFontText("2P (Need Joystick!)", 180, 165, FALSE);
	//DrawFontText("Config", 180, 201, FALSE);
	DrawFontText("Exit", 180, 201, FALSE);
	
	return;
}

void DrawMenu()
{
	//clear backbuffar
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	// Clear Screen And Depth Buffer
	glLoadIdentity();	

	DrawBG();
	
	if (SlidePos > TargetPos)
	{
		SlidePos -= 4;
	}
	else if (SlidePos < TargetPos)
	{
		SlidePos += 4;
	}
		
	DrawSelector();	
	DrawItems();
	
	//char out[50];
	//sprintf(out, "sin(%f) = %f", sineOsc, FloatOffset);	
	//DrawFontText(out, 100, ScreenHeight - 60, FALSE);
	
	return;
}

int DisplayMenu()
{
	// init menu stuff (textures etc)
	//FreeTextureData();
	LoadTextures();
	InitFont();
	
	StartMusic("snd/menu.ogg");
	// fade in?
	
	selection = 0;
	done = 0;
	SlidePos = 0;
	TargetPos = 0;
	
	while (done == 0)
	{
		// input
		ProcessInput();
		DrawMenu();
		SDL_GL_SwapBuffers();
		SDL_Delay(15);
	}
	
	// fade out?
	return selection;
}
