#include "SDL.h"
#include "SDL_mixer.h"
#include "physfs.h"
#include "physfsrwops.h"

#include "common.h"
#include "sound.h"

typedef struct {
	char* filename;
	int Active;
	Mix_Chunk *sound;
} SOUNDREF;

#define MAXSOUNDS 10

Mix_Music *music;
char WhatsPlaying[50];
SOUNDREF sounds[MAXSOUNDS];

void InitSound()
{
	int audio_rate = 22050;
	Uint16 audio_format = AUDIO_S16; /* 16-bit stereo */
	int audio_channels = 2;
	int audio_buffers = 4096;

	if( Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers)) {
	    sys_log("Unable to open audio!\n");
	    Quit(1);
	}
	
	int i;
	for (i = 0; i < MAXSOUNDS; i++)
	{
		sounds[i].Active = FALSE;
	}
	
	return;
}

int SoundLoaded(const char* filename)
{
	int i;
	for (i = 0; i < MAXSOUNDS; i++)
	{
		if ((sounds[i].Active == TRUE) && (strcmp(filename, sounds[i].filename) == 0))
			return i;
	}
	
	return -1;
}

void LoadSound(const char* filename)
{
	
	int indx = SoundLoaded(filename);
	if (indx != -1)
		// already loaded. bye
		return;
	
	sys_log("Loading sound %s: ...", filename);
		
	int i;
	for (i = 0; i < MAXSOUNDS; i++)
	{
		if (sounds[i].Active == FALSE)
		{
			indx = i;
			break;
		}
	}
	
	if (PHYSFS_exists(filename) == 0)
	{
            sys_log("Sound doesn't exist: %s\n", filename);
	    Quit(2);
	}
	
	//sys_log("ohno2 ...");
	
	SDL_RWops *rw;
	rw = PHYSFSRWOPS_openRead(filename);
		
	sounds[i].sound = Mix_LoadWAV_RW(rw, 0);
	sounds[i].filename = (char *)filename;
	sounds[i].Active = TRUE;
	
	sys_log("OK!\n");
	
	return;
}

void PlaySound(const char* filename)
{
	int indx = SoundLoaded(filename);
	if (indx == -1)
		//not loaded
		LoadSound(filename);
	
	if (sounds[indx].Active == TRUE)
		Mix_PlayChannel(-1, sounds[indx].sound, 0);
	
	return;
}

void StartMusic(const char* filename)
{
	if ((WhatsPlaying != NULL) && (strcmp(filename, WhatsPlaying) == 0))
		return;
	
	if (PHYSFS_exists(filename) == 0)
	{
		sys_log("Music doesn't exist: %s\n", filename);
	    Quit(2);
	}
	
	sys_log("Playing music: %s ... ", filename);
	
	SDL_RWops *rw;
	rw = PHYSFSRWOPS_openRead(filename);
	
	music = (Mix_Music *)Mix_LoadMUS_RW((SDL_RWops *)rw);
		
	if (!music)
		sys_log("\n%s\n", Mix_GetError());
	else
	{
		Mix_PlayMusic(music, -1);
		strcpy((char *)WhatsPlaying, filename);
		sys_log("OK!\n");
	}
	
	return;
}

void StopMusic()
{
	Mix_HaltMusic();
	Mix_FreeMusic(music);
	music = NULL;
	Mix_HaltChannel(-1);
	
	return;
}

void QuitSound()
{
	StopMusic();
	
	int i;
	
	for (i = 0; i < MAXSOUNDS; i++)
	{
		if (sounds[i].Active)
			Mix_FreeChunk(sounds[i].sound);
	}
	
	Mix_CloseAudio();
	
	return;
}
