#include <stdio.h>
#include <string.h>
#include "SDL.h"
#include "SDL_opengl.h"
#include "physfs.h"

#include "texture.h"
#include "common.h"

int numberOfRefsInArray = 0;
int	numberOfRefsAllocated = 0;

int GetRefsInArray()
{
	return numberOfRefsInArray;
}

int GetRefsAllocated()
{
	return numberOfRefsAllocated;
}

int TextureExistsInArray(const char* filename)
{
    int i = 0;

    for (i = 0; i < numberOfRefsInArray; i++)
    {
        //sys_log("%d\n", i);
        if (&CurrentRefs[i] != NULL)
        {
            if (strcmp(filename, CurrentRefs[i].filename) == 0)
            {
                return i;
            }
        }    
    }

    return -1;
}

void AddTextureToArray(TextureRef newTexture)
{ 
	if(numberOfRefsInArray == numberOfRefsAllocated) { // need more refs?

        if (numberOfRefsAllocated == 0)
		      numberOfRefsAllocated = 10; // start off with 2 refs
        else
		      numberOfRefsAllocated *= 2; // double the refs allocated
		CurrentRefs = (TextureRef*)realloc(CurrentRefs, (numberOfRefsAllocated * sizeof(TextureRef)));
	}

        CurrentRefs[numberOfRefsInArray] = newTexture; 

	numberOfRefsInArray++;
}

// TODO: Get this to work :D
int RemoveTexture(const char* filename)
{
    // Search through references..
    //int result = -1;
    int i;

    for (i = 0; i < numberOfRefsInArray; i++)
    {
		// If the filename matches remove the reference from the array
        if (filename == CurrentRefs[i].filename)
        {
			// TODO: Remove reference from the array
            return TRUE;
        }
    }
    
	// No filename matches
    return FALSE;
}

TextureRef LoadTGATexture(const char* filename)
{
    Texture *texture = (Texture*)malloc(sizeof(Texture));
    
    sys_log("Loading %s ... ", filename);

    // Targa data type -- uncompressed, RGBA image; element
    // 3 is always 2.
    GLubyte    targaMagic[12] = { 0, 0, 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 };

    GLubyte    fileMagic[12];

    // Header data.
    GLubyte    header[6];
    GLuint     bytesPerPixel;

    GLuint     imageSize;
    //GLint      sizeRead;
    GLuint     tmp;
    GLint      i;

    if (PHYSFS_exists(filename) == 0)
    {
    	sys_log("Texture doesn't exist: %s", filename);
        Quit(2);
    }

    PHYSFS_file* file;
    file = PHYSFS_openRead(filename);

    // Start reading in the header, make sure we
    // have the right Targa data type.  

    if(file == NULL || 
        PHYSFS_read(file, fileMagic, sizeof(fileMagic), 1) 	!= 1 	||
        memcmp(targaMagic, fileMagic, sizeof(targaMagic)) 	!= 0	||
        PHYSFS_read(file, header, sizeof(header), 1)		!= 1 )
    {
    	sys_log("Error loading texture: %s\n", filename);
        PHYSFS_close(file);
        Quit(2);
		//return FALSE;
    }
    //fprintf(stdout, "loaded ... ");
    // Determine width and height from header data.

    texture->width  = header[1] * 256 + header[0];
    texture->height = header[3] * 256 + header[2];

    // Determine color bit depth of the image (24 or 32).

    if(texture->width <= 0 || texture->height <= 0 || (header[4] != 24 && header[4] != 32))
    {
    	sys_log("Error loading texture: %s", filename);
        PHYSFS_close(file);
        Quit(2);
		//return FALSE;
    }
    
    texture->bpp  = header[4];
    bytesPerPixel = texture->bpp / 8;
    imageSize     = texture->width * texture->height * bytesPerPixel;
    //fprintf(stdout, "prepared (%d, %d) - %d... ", texture->width, texture->height, imageSize);
    
    texture->imageData = (GLubyte *)malloc(imageSize);
    
    //fprintf(stdout, "assigning ... ");
    
    if(texture->imageData == NULL || PHYSFS_read(file, texture->imageData, imageSize, 1) != 1)
    {
        if(texture->imageData != NULL)
            free(texture->imageData);

        sys_log("Error creating texture: %s", filename);
        PHYSFS_close(file);
        Quit(2);
		//return FALSE;
    } 
    //fprintf(stdout, "assigned ... ");
    // done reading, so close the file
    PHYSFS_close(file);

    for(i = 0; i < imageSize; i += bytesPerPixel)
    {
        tmp = texture->imageData[i];
        texture->imageData[i] = texture->imageData[i + 2];
        texture->imageData[i + 2] = tmp;
    }
    //fprintf(stdout, "referenced ... ");
    // Time to flip the image over!
    // This flipping snippet from http://gpwiki.org/index.php/LoadTGACpp
    unsigned char bTemp;
    unsigned char *pLine1, *pLine2;
    int iLineLen,iIndex;

    iLineLen=texture->width * (texture->bpp / 8);
    pLine1=texture->imageData;
    pLine2=&texture->imageData[iLineLen * (texture->height - 1)];

    for( ;pLine1<pLine2;pLine2-=(iLineLen*2))
    {
        for(iIndex=0;iIndex!=iLineLen;pLine1++,pLine2++,iIndex++)
        {
            bTemp=*pLine1;
            *pLine1=*pLine2;
            *pLine2=bTemp;
        }
    }
    //fprintf(stdout, "vflipped ... ");
    // Use OpenGL to load up the texture now
    GLuint tempref[1];

    // Create The Texture
    glGenTextures( 1, &tempref[0] );

    glBindTexture( GL_TEXTURE_2D, tempref[0] );

    // Generate The Texture
    glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, texture->width, texture->height, 0, GL_RGBA,
        GL_UNSIGNED_BYTE, texture->imageData );

    // Man, I love filtering
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	//fprintf(stdout, "texturated ... ");
    
    TextureRef ref;
    
    ref.textureID = tempref[0];
	//ref.filename = (char*)malloc(strlen(filename) * sizeof(char));
	//ref.filename = filename;
    ref.filename = (char *)(filename);

    ref.height = texture->height;
    ref.width = texture->width;
    ref.bpp = texture->bpp;

	free(texture->imageData);
    free(texture);

    sys_log("OK!\n");

    return ref;
}

TextureRef GetTexture(const char* filename, int ForceReload)
{
    // If texture is already loaded ... (and ForceReload is false)
    if ((numberOfRefsInArray > 0) && (ForceReload == FALSE))
    {
        int result = TextureExistsInArray(filename);
        // Return the texture reference

        if (result > -1)
            return CurrentRefs[result];

    }
    // Else
    TextureRef temp;

    // Load the texture
    temp = LoadTGATexture(filename);	

    // Add to array
    AddTextureToArray(temp);

    // Return the texture reference
    return temp;
}

void FreeTextureData()
{
	/*int i = 0;
    for (i = 0; i < numberOfRefsInArray; i++)
    {
        free(CurrentRefs[i].filename);
	}*/
	
	free(CurrentRefs);
}

